@@ -66,7 +66,12 @@ void BatchRenamer::processFiles( ProgressDialog* p, QObject* object )
     delete object;
     t.start();
 
-    for( unsigned int i = 0; i < m_files.count(); i++) {
+    m_counters.clear();
+
+    for( unsigned int i = 0; i < m_files.count(); i++) 
+    {
+	m_counter_index = 0;
+
         if( m_mode == RENAME ) {// final Path = source Path
             m_files[i].dst.directory = m_files[i].src.directory;
             m_files[i].dst.url = m_files[i].src.url;
@@ -78,7 +83,12 @@ void BatchRenamer::processFiles( ProgressDialog* p, QObject* object )
 
         if( i == 0 )
             p->setDestination( m_files[i].dst.url );
-
+	else
+	{
+	    if( m_reset )
+		findCounterReset( i );
+	}
+	
         m_files[i].dst.name = processString( text, m_files[i].src.name, i );
         if( !extext.isEmpty() )
             m_files[i].dst.extension = processString( extext, m_files[i].src.extension, i );
@@ -112,8 +122,6 @@ void BatchRenamer::processFiles( ProgressDialog* p, QObject* object )
             }
         }
     }
-    m_counter_index = 0;
-    m_skip_add.clear();
 
     p->print( QString( i18n("Filenames Processed after %1 seconds.")).arg(t.elapsed()/1000) );
 
@@ -138,7 +146,7 @@ QString BatchRenamer::processString( QString text, QString oldname, int i )
      */
      
 
-    oldname = doEscape( oldname );
+    doEscape( oldname );
     /*
      * Call here all functions that handle
      * arguments that are single tokens (&,%,...).
@@ -171,7 +179,7 @@ QString BatchRenamer::processString( QString text, QString oldname, int i )
     return text;
 }
 
-QString BatchRenamer::parsePlugins( int i, QString text, int type )
+QString BatchRenamer::parsePlugins( int i, const QString& text, int type )
 {
     QPtrListIterator<PluginLoader::PluginLibrary> it( plug->libs );
     QString ret = text;
@@ -179,10 +187,12 @@ QString BatchRenamer::parsePlugins( int i, QString text, int type )
     if( type == TYPE_FINAL_FILE )
         ret = "";
         
-    for( ; it.current(); ++it ) {
+    for( ; it.current(); ++it )
         if( (*it)->usePlugin && (*it)->plugin->type() == type )
-            ret = doEscape( (*it)->plugin->processFile( this, i, text, type ) );
-    }
+        {
+            ret = (*it)->plugin->processFile( this, i, text, type );
+            doEscape( ret );
+        }
 
     return ret;
 }
@@ -192,10 +202,14 @@ void BatchRenamer::createPreview( QListView* list )
     KMyListViewItem* item1 = NULL;
     QString tmp;
 
-    m_counter_index = 0;
-    m_skip_add.clear();
+    m_counters.clear();
     for( unsigned int i = 0; i < m_files.count(); i++) 
     {            
+	m_counter_index = 0;
+
+	if( i && m_reset )
+	    findCounterReset( i );
+
         m_files[i].dst.name = processString( text, m_files[i].src.name, i );
         if( !extext.isEmpty() )
             m_files[i].dst.extension = processString( extext, m_files[i].src.extension, i );
@@ -317,7 +331,10 @@ void BatchRenamer::work( ProgressDialog* p )
 
     const QString m = QString( i18n("Renamed %1 files successfully.") ).arg(i-error);
     ( i - error ) ? p->print( m ) : p->warning( m );
-    if( i - error > 0 ) p->warning( QString( i18n("%2 errors occurred!") ).arg(error));
+
+    if( error > 0 ) 
+        p->warning( QString( i18n("%2 errors occurred!") ).arg(error));
+
     p->print( QString( i18n("Elapsed time: %1 seconds") ).arg( t.elapsed()/1000 ), "kalarm" );
     p->print( i18n("KRename finished the renaming process."), "krename" );
     p->print( i18n("Press close to quit!") );
@@ -339,7 +356,7 @@ void BatchRenamer::work( ProgressDialog* p )
     delete this;
 }
 
-QString BatchRenamer::escape( QString text, QString token, QString sequence )
+void BatchRenamer::escape( QString & text, const QString & token, const QString & sequence )
 {
     /*
      * NEVER, ABSOLUTELY NEVER change pos = 0
@@ -357,54 +374,53 @@ QString BatchRenamer::escape( QString text, QString token, QString sequence )
            text.replace( pos, token.length(), sequence );
     } while ( pos >= 0 );
 #endif
-    return text;
 }
 
-QString BatchRenamer::doEscape( QString text, bool filename )
+QString & BatchRenamer::doEscape( QString & text, bool filename )
 {
     if( filename ) {
-        text = BatchRenamer::escape( text, "&", QChar( 60000 ) );
-        text = BatchRenamer::escape( text, "$", QChar( 60001 ) );
-        text = BatchRenamer::escape( text, "%", QChar( 60002 ) );
-        text = BatchRenamer::escape( text, "#", QChar( 60004 ) );
-        text = BatchRenamer::escape( text, "[", QChar( 60005 ) );
-        text = BatchRenamer::escape( text, "]", QChar( 60006 ) );
-        text = BatchRenamer::escape( text, "\\", QChar( 60007 ) );
-        text = BatchRenamer::escape( text, "/", QChar( 60008 ) );
-        text = BatchRenamer::escape( text, "{", QChar( 60009 ) );        
-        text = BatchRenamer::escape( text, "}", QChar( 60010 ) );        
+        BatchRenamer::escape( text, "&", QChar( 60000 ) );
+        BatchRenamer::escape( text, "$", QChar( 60001 ) );
+        BatchRenamer::escape( text, "%", QChar( 60002 ) );
+        BatchRenamer::escape( text, "#", QChar( 60004 ) );
+        BatchRenamer::escape( text, "[", QChar( 60005 ) );
+        BatchRenamer::escape( text, "]", QChar( 60006 ) );
+        BatchRenamer::escape( text, "\\", QChar( 60007 ) );
+        BatchRenamer::escape( text, "/", QChar( 60008 ) );
+        BatchRenamer::escape( text, "{", QChar( 60009 ) );        
+        BatchRenamer::escape( text, "}", QChar( 60010 ) );        
     } else {
-        text = BatchRenamer::escape( text, "\\&", QChar( 60000 ) );
-        text = BatchRenamer::escape( text, "\\$", QChar( 60001 ) );
-        text = BatchRenamer::escape( text, "\\%", QChar( 60002 ) );
-        text = BatchRenamer::escape( text, "\\#", QChar( 60004 ) );
-        text = BatchRenamer::escape( text, "\\[", QChar( 60005 ) );
-        text = BatchRenamer::escape( text, "\\]", QChar( 60006 ) );
-        text = BatchRenamer::escape( text, "\\", QChar( 60007 ) ); 
-        text = BatchRenamer::escape( text, "\\/", QChar( 60008 ) );
-        text = BatchRenamer::escape( text, "\\{", QChar( 60009 ) );        
-        text = BatchRenamer::escape( text, "\\}", QChar( 60010 ) );        
+        BatchRenamer::escape( text, "\\&", QChar( 60000 ) );
+        BatchRenamer::escape( text, "\\$", QChar( 60001 ) );
+        BatchRenamer::escape( text, "\\%", QChar( 60002 ) );
+        BatchRenamer::escape( text, "\\#", QChar( 60004 ) );
+        BatchRenamer::escape( text, "\\[", QChar( 60005 ) );
+        BatchRenamer::escape( text, "\\]", QChar( 60006 ) );
+        BatchRenamer::escape( text, "\\\\", QChar( 60007 ) ); 
+        BatchRenamer::escape( text, "\\/", QChar( 60008 ) );
+        BatchRenamer::escape( text, "\\{", QChar( 60009 ) );        
+        BatchRenamer::escape( text, "\\}", QChar( 60010 ) );        
     }
 
     return text;
 }
 
-QString BatchRenamer::unEscape( QString text )
+QString & BatchRenamer::unEscape( QString & text )
 { 
-    text = BatchRenamer::escape( text, QChar( 60000 ), "&" );
-    text = BatchRenamer::escape( text, QChar( 60001 ), "$" );
-    text = BatchRenamer::escape( text, QChar( 60002 ), "%" );
-    text = BatchRenamer::escape( text, QChar( 60004 ), "#" );
-    text = BatchRenamer::escape( text, QChar( 60005 ), "[" );
-    text = BatchRenamer::escape( text, QChar( 60006 ), "]" );
-    text = BatchRenamer::escape( text, QChar( 60007 ), "\\" );
+    BatchRenamer::escape( text, QChar( 60000 ), "&" );
+    BatchRenamer::escape( text, QChar( 60001 ), "$" );
+    BatchRenamer::escape( text, QChar( 60002 ), "%" );
+    BatchRenamer::escape( text, QChar( 60004 ), "#" );
+    BatchRenamer::escape( text, QChar( 60005 ), "[" );
+    BatchRenamer::escape( text, QChar( 60006 ), "]" );
+    BatchRenamer::escape( text, QChar( 60007 ), "\\" );
     // %252f == /, it seems that filenames on unix cannot contain
     // a /. So I use %252f, at least konqui displays it correctly
     // this was needed, so that plugins that return a slash do not cause errors
-    text = BatchRenamer::escape( text, QChar( 60008 ), "%2f" );
-    text = BatchRenamer::escape( text, QChar( 60009 ), "\\{" );        
-    text = BatchRenamer::escape( text, QChar( 60010 ), "\\}" );        
-    
+    BatchRenamer::escape( text, QChar( 60008 ), "%2f" );
+    BatchRenamer::escape( text, QChar( 60009 ), "\\{" );        
+    BatchRenamer::escape( text, QChar( 60010 ), "\\}" );        
+
     return text;
 }
 
@@ -415,7 +431,7 @@ int BatchRenamer::getCharacters( int n )
     return s.length();
 }
 
-QString BatchRenamer::findAndProcess( QString token, QString text, QString replace )
+QString BatchRenamer::findAndProcess( const QString & token, QString text, const QString & replace )
 {
     /*
      * pos can here be -1 because
@@ -441,14 +457,12 @@ QString BatchRenamer::findNumbers( QString text, int count, int i )
     // Added numbers skipping in 1.3
     // Changed again in Version 1.8 to optimize it and fix a bug with skipping numbers
     int pos = 0, counter = 1;
-    int start = m_index;
-    int step  = m_step;
-    
+    tCounterValues countervalues;
+    countervalues.start = m_index;
+    countervalues.step = m_step;
+
     if( text.contains( "#", FALSE ) <= 0 )
-    {
-        m_counter_index = 0;
         return text;
-    } 
 
     pos = text.find("#", pos);
     pos++;
@@ -457,12 +471,23 @@ QString BatchRenamer::findNumbers( QString text, int count, int i )
         counter++;
     }
 
-    findNumberAppendix( text, pos, &start, &step );
+    findNumberAppendix( text, pos, &countervalues.start, &countervalues.step );
 
     pos = text.find("#", 0);
-    if( (signed int)m_skip_add.count() <= m_counter_index )
-        m_skip_add.append( 0 );
+
+    if( (signed int)m_counters.count() <= m_counter_index )
+    {
+	countervalues.value = countervalues.start - countervalues.step;
+	// other wise the counter would start at:
+	// start + step instead of start
+	m_counters.append( countervalues );
+    }
+
+    do {
+	m_counters[m_counter_index].value += m_counters[m_counter_index].step;
+    } while( m_skip.contains( m_counters[m_counter_index].value ) );
     
+    /*
     int v = start + (i*step) + m_skip_add[m_counter_index];
     for( unsigned int z = 0; z < m_skip.count(); z++ ) {
         if( m_skip[z] == v ) {
@@ -470,9 +495,10 @@ QString BatchRenamer::findNumbers( QString text, int count, int i )
             v += step;
         }
     }
-    
+    */
+
     QString temp;
-    temp.sprintf("%0*i", counter, v );
+    temp.sprintf("%0*i", counter, m_counters[m_counter_index].value );
     text.replace( pos, 1, temp);
 
     ++m_counter_index;
@@ -502,7 +528,7 @@ void BatchRenamer::findNumberAppendix( QString & text, int pos, int* start, int*
     }
 }
 
-QString BatchRenamer::findStar( QString oldname, QString text )
+QString BatchRenamer::findStar( const QString & oldname, QString text )
 {
     int pos = -1;
     do {
@@ -796,8 +822,9 @@ QString BatchRenamer::doReplace( QString text, QString find, QString replace, bo
 #endif
     } else {
 #if QT_VERSION >= 0x030100    
-        // no doEscape() her, because it would destroy our regular expression
-        text.replace( QRegExp( find ), replace );
+        // no doEscape() here for the regexp, because it would destroy our regular expression
+        // other wise we will not find stuff like $, [ in the text 
+        text = doEscape( unEscape( text ).replace( QRegExp( find ), replace ) );
 #else
         // Test this code more!
         pos = 0;
@@ -805,7 +832,7 @@ QString BatchRenamer::doReplace( QString text, QString find, QString replace, bo
             QRegExp exp( find );
             pos = exp.search( text, pos );
             if( pos >= 0 ) {
-                text.replace( pos, exp.matchedLength(), replace );
+                text = doEscape( unEscape( text ).replace( pos, exp.matchedLength(), replace ) );
                 pos += replace.length();
             }
         } while( pos >= 0 );
@@ -871,8 +898,10 @@ void BatchRenamer::parseSubdirs( data* f )
 
 QString BatchRenamer::buildFilename( fileentry* entry, bool dir )
 {
+    QString filename = ( dir ? entry->directory : QString::null ) + entry->name + ( entry->extension.isEmpty() ? QString::null : QString(".") ) + entry->extension;
     // unescape here as filename is still escaped
-    return unEscape( ( dir ? entry->directory : QString::null ) + entry->name + ( entry->extension.isEmpty() ? QString::null : QString(".") ) + entry->extension );
+    unEscape( filename );
+    return filename;
 }
 
 bool BatchRenamer::applyManualChanges( int i )
@@ -897,3 +926,14 @@ bool BatchRenamer::applyManualChanges( int i )
     return false;
 }
 
+void BatchRenamer::findCounterReset( int i )
+{
+    int z;
+    if( m_files[i-1].src.directory != m_files[i].src.directory )
+	for( z=0;z<(int)m_counters.count();z++ )
+	{
+	    m_counters[z].value = m_counters[z].start - m_counters[z].step;
+	}
+}
+
+
