@@ -45,6 +45,8 @@ using namespace KIO;
 KMyListBox::KMyListBox(QWidget* parent, const char* name, WFlags fl)
     :KListBox(parent, name, fl)
 {
+    m_running_lister_counter = 0;
+
     drag = ctrlPressed = shiftPressed = mousePressed = false;
     moving = false;
     m_sorting = UNSORTED;
@@ -97,6 +99,7 @@ void KMyListBox::addFile( const KURL & filename, bool isfile )
         return;
 
     insertItem( new KMyListBoxItem( filename, false ), -1 );
+
     positionLabel();
 }
 
@@ -109,7 +112,7 @@ void KMyListBox::addDirName( const KURL & dirname )
     }
 }
 
-bool KMyListBox::isFile( KURL f, bool autoadd )
+bool KMyListBox::isFile( const KURL & f, bool autoadd )
 {
     UDSEntry entry;
     
@@ -127,9 +130,9 @@ bool KMyListBox::isFile( KURL f, bool autoadd )
     return true;
 }
 
-void KMyListBox::addDir( KURL dirname, const QString & filter, bool hidden, bool recursively, bool dirnames )
+void KMyListBox::addDir( const KURL & dirname, const QString & filter, bool hidden, bool recursively, bool dirnames )
 {
-    ThreadedLister* thl = new ThreadedLister( &m_add_mutex, this );
+    ThreadedLister* thl = new ThreadedLister( &m_add_mutex, &m_running_lister_counter, this );
     thl->setDirname( dirname );
     thl->setDirnames( dirnames );
     thl->setFilter( filter );
@@ -140,11 +143,12 @@ void KMyListBox::addDir( KURL dirname, const QString & filter, bool hidden, bool
     thl->start();
 }
 
-void KMyListBox::addDirName( KURL dirname, const QString & filter, bool hidden, bool recursive )
+void KMyListBox::addDirName( const KURL & dirname, const QString & filter, bool hidden, bool recursive )
 {
     KApplication::setOverrideCursor( Qt::waitCursor );            
+
     if( recursive ) {
-        ThreadedLister* thl = new ThreadedLister( &m_add_mutex, this );
+        ThreadedLister* thl = new ThreadedLister( &m_add_mutex, &m_running_lister_counter, this );
         thl->setDirname( dirname );
         thl->setDirnames( true );
         thl->setFilter( filter );
@@ -174,6 +178,7 @@ void KMyListBox::dropEvent(QDropEvent* e)
     {
         
         KApplication::setOverrideCursor( Qt::waitCursor );
+
         setUpdatesEnabled( false );
 
         for( unsigned int i = 0; i < list.count(); i++ )
@@ -518,6 +523,17 @@ void KMyListBox::listerDone( ThreadedLister* lister )
     m_add_mutex.unlock(); 
 }
 
+unsigned int KMyListBox::runningAddListeners()
+{ 
+    unsigned int u;
+
+    m_add_mutex.lock();
+    u = m_running_lister_counter;
+    m_add_mutex.unlock(); 
+
+    return u;
+}
+
 void KMyListBox::sortAscending()
 {
     m_sorting = ASCENDING;
@@ -712,6 +728,13 @@ void KMyListBox::positionLabel()
     }
 }
 
+void KMyListBox::paintEvent( QPaintEvent* e )
+{
+    //   qDebug("Updates=%i", (int)isUpdatesEnabled() );
+    //if( isUpdatesEnabled() )
+        KListBox::paintEvent( e );
+}
+
 KMyListBoxItem::KMyListBoxItem( const KMyListBoxItem* item )
     : QListBoxItem()
 {
