@@ -92,6 +92,8 @@ KRenameImpl::KRenameImpl( QWidget* p, KMenuBar* m, QPushButton* finish )
     : DCOPObject( "KRename" ), QObject( (QObject*)p ),
       parent( p ), menuBar( m ), finishButton( finish )
 {
+    m_hasCommandlineProfile = false;
+
     // Load Plugins
     plugin = PluginLoader::instance();
     helpDialogData = new HelpDialogData();
@@ -111,7 +113,7 @@ KRenameImpl::~KRenameImpl()
 
 bool KRenameImpl::noRealTimePreview = false;
 
-void KRenameImpl::setup()
+void KRenameImpl::setup( bool wizardmode )
 {
     parent->setCaption( "KRename" ); // "KRename by Dominik Seichter
     setupActions();
@@ -168,7 +170,11 @@ void KRenameImpl::setup()
     filename->setText( "$" );   // no i18n() ! ;)
     extemplate->setText( "$" );
 
+    // do it now so that it can be overwritten by commandline args
+    setWizardMode( wizardmode );
+
     parseCommandline();
+    enableControls();
 }
 
 const QString KRenameImpl::title( int index ) const
@@ -183,48 +189,67 @@ void KRenameImpl::parseCommandline()
     QCStringList optlist = args->getOptionList ( "r" );
     for (QCStringList::ConstIterator it=optlist.begin(); it!=optlist.end(); ++it)
     {
+
         KURL url;
         url.setPath( *it );
         fileList->addDir( url, "*", false, true, false );
     }
-    
+
+    //fileList->setUpdatesEnabled( false );
+    //fileList->viewport()->setUpdatesEnabled( false );
     for( int i = 0; i < args->count(); i++)
         fileList->addFile( args->url( i ) );
+    //fileList->viewport()->setUpdatesEnabled( true );
+    //fileList->setUpdatesEnabled( true );
+
+    // load the profile first, so that we do not overwrite other
+    // commandline settings
+    QCString templ = args->getOption( "profile" );
+    if( !templ.isEmpty() )
+    {
+        m_hasCommandlineProfile = true;
+        ProfileManager::loadProfile( QString( templ ), this );
+    }
 
     if( !args->isSet( "preview" ) )
         noRealTimePreview = true;
     else
         noRealTimePreview = false;
 
-    QCString templ = args->getOption( "template" );
+    templ = args->getOption( "template" );
     if( !templ.isEmpty() )
         filename->setText( templ );
 
     templ = args->getOption( "extension" );
-    if( !templ.isEmpty() ) {
+    if( !templ.isEmpty() ) 
+    {
         extemplate->setText( templ );
         checkExtension->setChecked( false );
     }
 
     templ = args->getOption( "copy" );
-    if( !templ.isEmpty() ) {
+    if( !templ.isEmpty() ) 
+    {
         urlrequester->setURL( templ );
         optionCopy->setChecked( true );
     }
 
     templ = args->getOption( "move" );
-    if( !templ.isEmpty() ) {
+    if( !templ.isEmpty() ) 
+    {
         urlrequester->setURL( templ );
         optionMove->setChecked( true );
     }
         
     QCStringList uselist = args->getOptionList ( "use-plugin" );
-    if( !uselist.isEmpty() ) {
+    if( !uselist.isEmpty() ) 
+    {
         for(unsigned int i = 0; i < uselist.count(); i++ )
             uselist[i] = uselist[i].lower();
     
         QPtrListIterator<PluginLoader::PluginLibrary> it( plugin->libs );
-        while ( it.current() ) {
+        while ( it.current() ) 
+        {
             if( uselist.contains( (*it)->plugin->getName().lower().utf8() ) )
                 (*it)->check->setChecked( true );
                 
@@ -250,10 +275,19 @@ void KRenameImpl::parseCommandline()
         emit showPage( m_wizard ? 3 : 4 );
     }
 
-    if( startnow && fileList->count() ) {
-        start();
-        // hide the KRename main window after renaming:
-        QTimer::singleShot( 200, parent, SLOT( hide() ) );
+    if( startnow ) 
+    {
+        // As file adding runs in a another trhread,
+        // there might be adding in progress but not yet
+        // all files in the list.
+        // so let's wait for file adding to finish first
+        // before starting.
+        while( fileList->runningAddListeners() > 0 )
+            kapp->processEvents();
+
+        if( fileList->count() ) 
+            // start renaming
+            QTimer::singleShot( 200, this, SLOT( start() ) );
     }
 }
 
@@ -465,7 +499,6 @@ void KRenameImpl::setupPage4()
     setupFileTab1();
     setupFileTab2();
     
-    Layout21 = new QVBoxLayout( 0, 0, 6 );
     Layout22 = new QHBoxLayout( 0, 0, 6 );
     Layout23 = new QVBoxLayout( 0, 0, 6 );
     
@@ -533,7 +566,7 @@ void KRenameImpl::setupFileTab1()
 
     comboExtension = new KComboBox( false, tab );
 
-    buttonReplace = new KPushButton( i18n("&Find and Replace..."), tab );
+    buttonReplace = new KPushButton( i18n("Find &and Replace..."), tab );
     buttonReplace->setIconSet( SmallIconSet("find") );
 
     buttonNumber = new KPushButton( i18n("&Numbering..."), tab );
@@ -845,17 +878,15 @@ void KRenameImpl::setupTab1()
     QVBoxLayout* Layout;
     QVBoxLayout* gLayout;
     QGroupBox* g;
+    QWidget* page;
 
     /* strangely I do not have to delete this stuff
      * after plugins have been reloaded.
      * could cause a memory leak though
-    int index = 0;    
-    while( ( page = page_3->pageWidget( index ) ) )
-    {
+     */
+    while( ( page = page_3->pageWidget( 0 ) ) )
         page_3->removePage( page );
-        index++;
-    }*/
-    
+
     QPtrListIterator<PluginLoader::PluginLibrary> it( plugin->libs ); // iterator for dict
     while ( it.current() ) {
         if( !(*it)->plugin->alwaysUsed() )
@@ -872,11 +903,14 @@ void KRenameImpl::setupTab1()
             (*it)->plugin->drawInterface( g, gLayout );
 
             Layout->addWidget( g );
+            
         } else
             (*it)->plugin->addHelp( helpDialogData );
             
         ++it;
     }
+
+    page_3->showPage( 1 );
 }
 
 void KRenameImpl::addFile()
@@ -887,12 +921,11 @@ void KRenameImpl::addFile()
     if( dsd->exec() == QDialog::Accepted ) {
         KURL::List slist = dsd->selectedURLs();
         KURL::List::Iterator it = slist.begin();
+
         for ( ; it != slist.end(); ++it )
         {
             if( !fileList->isFile( *it, false ) )
             {
-/*                qDebug("DSD-onlyDirs=%i", dsd->onlyDirs());
-                qDebug("DST-dirs=%i", dsd->dirs() );*/
                 if( dsd->onlyDirs() )
                     fileList->addDirName( *it, dsd->currentFilter(), dsd->hidden(), dsd->recursively() );
                 else
@@ -1087,7 +1120,7 @@ void KRenameImpl::fillStructures( BatchRenamer* b )
      */
     QPtrListIterator<PluginLoader::PluginLibrary> it( plugin->libs );
     for( int z = 0; it.current(); ++it, z++ ) {
-        if( (*it)->plugin->alwaysUsed() || (*it)->check->isChecked() )
+        if( (*it)->plugin->alwaysUsed() || ((*it)->check && (*it)->check->isChecked()) )
             (*it)->plugin->fillStructure();
     }
 
@@ -1101,12 +1134,12 @@ void KRenameImpl::updatePre()
     toggleImagePreview();
 }
 
-void KRenameImpl::addFileOrDir( QString name )
+void KRenameImpl::addFileOrDir( const QString & name )
 {
     fileList->addFile( name, false );
 }
 
-void KRenameImpl::addFileOrDir( KURL name )
+void KRenameImpl::addFileOrDir( const KURL & name )
 {
     fileList->addFile( name, true );
 }
@@ -1116,6 +1149,7 @@ bool KRenameImpl::setupBatchRenamer( BatchRenamer* b, bool preview )
     b->setText( filename->text() );
     b->setExText( ( checkExtension->isChecked() ? QString("$") : extemplate->text() ) );
     b->setIndex( m_index );
+    b->setResetCounter( m_reset );
     b->setStep( m_step );
     b->setSkipList( skip );
     b->setReplaceList( rep );
@@ -1220,6 +1254,7 @@ void KRenameImpl::loadConfig()
     fileList->setPreview( checkPreview->isChecked() );
     m_index = config->readNumEntry("StartIndex", 1);
     m_step = 1;
+    m_reset = false;
     m_loadplugins = config->readBoolEntry("LoadFilePlugins", true );
     m_autosize = config->readBoolEntry("ColumnAutosize", false );
     checkName->setChecked(config->readBoolEntry("Name", false ));
@@ -1258,7 +1293,7 @@ void KRenameImpl::loadConfig()
         checkName->setEnabled( false );
     }
 
-    parent->show();
+    //parent->show();
     page_1->setEnabled( false );
     KApplication::setOverrideCursor( Qt::WaitCursor );
     plugin->loadPlugins( m_loadplugins );
@@ -1373,13 +1408,14 @@ void KRenameImpl::changeGUIMode()
     int h = parent->height();
 
     QWidget* oldparent = parent;
-    QWidget* krename = KRenameImpl::launch( QRect( x, y, w, h ), QStringList(), this );
+    QWidget* krename = KRenameImpl::launch( QRect( x, y, w, h ), QStringList(), this, false );
     
     m_switching = true;
     oldparent->removeChild( this );
     oldparent->hide();
     krename->insertChild( this );
     krename->show();
+
     oldparent->close();
     m_switching = false;            
 }
@@ -1433,9 +1469,13 @@ void KRenameImpl::pluginHelpChanged()
     while ( it.current() ) {
         if( !(*it)->plugin->alwaysUsed() ) {
             (*it)->plugin->removeHelp( helpDialogData );
-            (*it)->usePlugin = (*it)->check->isChecked();
-            if( (*it)->check->isChecked() )
-                (*it)->plugin->addHelp( helpDialogData );
+
+            if( (*it)->check )
+            {
+                (*it)->usePlugin = (*it)->check->isChecked();
+                if( (*it)->check->isChecked() )
+                    (*it)->plugin->addHelp( helpDialogData );
+            }
         } 
         else
         {
@@ -1539,12 +1579,14 @@ void KRenameImpl::changeNumbers()
     NumberDialog* nd = new NumberDialog( skip, parent );
     nd->spinIndex->setValue( m_index );
     nd->spinStep->setValue( m_step );
-    
+    nd->checkResetCounter->setChecked( m_reset );
+
     if( nd->exec() ) {
         m_index = nd->spinIndex->value();
         m_step = nd->spinStep->value();
         skip = nd->getList();
-        
+        m_reset = nd->checkResetCounter->isChecked();
+
         updatePreview();
     }
 }
@@ -1603,7 +1645,7 @@ void KRenameImpl::changeParent( QWidget* p, KMenuBar* m, QPushButton* finish, QR
     emit pageDone( page_4, i18n( pageTitle[3] ) );
 }
 
-QWidget* KRenameImpl::launch( QRect rect, const KURL::List & list, KRenameImpl* impl )
+QWidget* KRenameImpl::launch( QRect rect, const KURL::List & list, KRenameImpl* impl, bool loadprofile )
 {
     KConfig* config = kapp->config();
     config->setGroup("GUISettings");
@@ -1642,6 +1684,12 @@ QWidget* KRenameImpl::launch( QRect rect, const KURL::List & list, KRenameImpl*
 
     k->updatePre();
 
+    // it is time to load a default profile now (if the user has specified one)
+    if( loadprofile && !k->hasCommandlineProfile() && ProfileManager::hasDefaultProfile() )
+	ProfileManager::loadDefaultProfile( k );
+    else if ( !k->hasCommandlineProfile() )
+        w->show();
+
     return w;
 }
 
@@ -1726,13 +1774,10 @@ void KRenameImpl::setCounterStart( int index )
 
 void KRenameImpl::loadFilePlugins()
 {
-    if( !plugin->filePluginsLoaded() )
-    {
-        KApplication::setOverrideCursor( Qt::WaitCursor );        
-        plugin->loadPlugins( true );
-        KApplication::restoreOverrideCursor();
-        pluginHelpChanged();
-    }
+    KApplication::setOverrideCursor( Qt::WaitCursor );        
+    plugin->loadPlugins( true );
+    KApplication::restoreOverrideCursor();
+    pluginHelpChanged();
     
     enableControls();
 }
